@@ -62,6 +62,7 @@ hb_buffer_ensure_separate (hb_buffer_t *buffer, unsigned int size)
   hb_buffer_ensure (buffer, size);
   if (buffer->out_string == buffer->in_string)
   {
+    assert (buffer->have_output);
     if (!buffer->positions)
       buffer->positions = calloc (buffer->allocated, sizeof (buffer->positions[0]));
 
@@ -112,6 +113,7 @@ hb_buffer_destroy (hb_buffer_t *buffer)
 void
 hb_buffer_clear (hb_buffer_t *buffer)
 {
+  buffer->have_output = FALSE;
   buffer->in_length = 0;
   buffer->out_length = 0;
   buffer->in_pos = 0;
@@ -183,6 +185,7 @@ hb_buffer_set_direction (hb_buffer_t    *buffer,
 void
 _hb_buffer_clear_output (hb_buffer_t *buffer)
 {
+  buffer->have_output = TRUE;
   buffer->out_length = 0;
   buffer->out_pos = 0;
   buffer->out_string = buffer->in_string;
@@ -192,6 +195,7 @@ void
 hb_buffer_clear_positions (hb_buffer_t *buffer)
 {
   _hb_buffer_clear_output (buffer);
+  buffer->have_output = FALSE;
 
   if (HB_UNLIKELY (!buffer->positions))
   {
@@ -207,6 +211,8 @@ _hb_buffer_swap (hb_buffer_t *buffer)
 {
   unsigned int tmp;
 
+  assert (buffer->have_output);
+
   if (buffer->out_string != buffer->in_string)
   {
     hb_internal_glyph_info_t *tmp_string;
@@ -318,6 +324,12 @@ _hb_buffer_add_output_glyph (hb_buffer_t *buffer,
 void
 _hb_buffer_next_glyph (hb_buffer_t *buffer)
 {
+  if (!buffer->have_output)
+  {
+    buffer->in_pos++;
+    return;
+  }
+
   if (buffer->out_string != buffer->in_string)
   {
     hb_buffer_ensure (buffer, buffer->out_pos + 1);
@@ -362,7 +374,7 @@ hb_buffer_get_glyph_infos (hb_buffer_t *buffer)
 hb_glyph_position_t *
 hb_buffer_get_glyph_positions (hb_buffer_t *buffer)
 {
-  if (buffer->in_length && !buffer->positions)
+  if (buffer->have_output || (buffer->in_length && !buffer->positions))
     hb_buffer_clear_positions (buffer);
 
   return (hb_glyph_position_t *) buffer->positions;
@@ -423,7 +435,7 @@ hb_utf8_next (const uint8_t *text,
   unsigned int mask, len;
 
   UTF8_COMPUTE (c, mask, len);
-  if (HB_UNLIKELY (!len || end - text < len)) {
+  if (HB_UNLIKELY (!len || (unsigned int) (end - text) < len)) {
     *unicode = -1;
     return text + 1;
   } else {
