@@ -222,10 +222,13 @@ _hb_ot_layout_set_glyph_class (hb_face_t                  *face,
 
   /* XXX this is not threadsafe */
   if (glyph >= len) {
-    int new_len;
+    unsigned int new_len;
     unsigned char *new_klasses;
 
     new_len = len == 0 ? 120 : 2 * len;
+    while (new_len <= glyph)
+      new_len *= 2;
+
     if (new_len > 65536)
       new_len = 65536;
     new_klasses = (unsigned char *) realloc (layout->new_gdef.klasses, new_len * sizeof (unsigned char));
@@ -456,12 +459,13 @@ hb_ot_layout_language_get_feature_tags (hb_face_t    *face,
   const LangSys &l = g.get_script (script_index).get_lang_sys (language_index);
 
   ASSERT_STATIC (sizeof (unsigned int) == sizeof (hb_tag_t));
-  unsigned int count = feature_count ? *feature_count : 0;
   hb_bool_t ret = l.get_feature_indexes (feature_count, (unsigned int *) feature_tags);
 
-  count = feature_count ? MIN (count, *feature_count) : 0;
-  for (unsigned int i = 0; i < count; i++)
-    feature_tags[i] = g.get_feature_tag ((unsigned int) feature_tags[i]);
+  if (feature_tags) {
+    unsigned int count = *feature_count;
+    for (unsigned int i = 0; i < count; i++)
+      feature_tags[i] = g.get_feature_tag ((unsigned int) feature_tags[i]);
+  }
 
   return ret;
 }
