@@ -49,11 +49,9 @@ typedef struct _PangoContextClass PangoContextClass;
 
 GType         pango_context_get_type      (void) G_GNUC_CONST;
 
-#ifdef PANGO_ENABLE_BACKEND
 PangoContext *pango_context_new           (void);
 void          pango_context_set_font_map  (PangoContext                 *context,
 					   PangoFontMap                 *font_map);
-#endif /* PANGO_ENABLE_BACKEND */
 PangoFontMap *pango_context_get_font_map  (PangoContext                 *context);
 
 void          pango_context_list_families (PangoContext                 *context,
@@ -82,6 +80,9 @@ void                      pango_context_set_base_gravity     (PangoContext
 							      PangoGravity                gravity);
 PangoGravity              pango_context_get_base_gravity     (PangoContext               *context);
 PangoGravity              pango_context_get_gravity          (PangoContext               *context);
+void                      pango_context_set_gravity_hint     (PangoContext               *context,
+							      PangoGravityHint            hint);
+PangoGravityHint          pango_context_get_gravity_hint     (PangoContext               *context);
 
 void                        pango_context_set_matrix (PangoContext      *context,
 						      const PangoMatrix *matrix);
