@@ -24,26 +24,26 @@
 #include "pango-ot-private.h"
 #include "pangofc-private.h"
 
-/* cache a single HB_Buffer */
-static HB_Buffer cached_buffer = NULL;
+/* cache a single hb_buffer_t */
+static hb_buffer_t *cached_buffer = NULL;
 G_LOCK_DEFINE_STATIC (cached_buffer);
 
-static HB_Buffer
+static hb_buffer_t *
 acquire_buffer (gboolean *free_buffer)
 {
-  HB_Buffer buffer;
+  hb_buffer_t *buffer;
 
   if (G_LIKELY (G_TRYLOCK (cached_buffer)))
     {
       if (G_UNLIKELY (!cached_buffer))
-	hb_buffer_new (&cached_buffer);
+	cached_buffer = hb_buffer_create (64);
 
       buffer = cached_buffer;
       *free_buffer = FALSE;
     }
   else
     {
-      hb_buffer_new (&buffer);
+      buffer = hb_buffer_create (32);
       *free_buffer = TRUE;
     }
 
@@ -51,15 +51,15 @@ acquire_buffer (gboolean *free_buffer)
 }
 
 static void
-release_buffer (HB_Buffer buffer, gboolean free_buffer)
+release_buffer (hb_buffer_t *buffer, gboolean free_buffer)
 {
-  if (G_LIKELY (!free_buffer))
+  if (G_LIKELY (!free_buffer) && hb_buffer_get_reference_count (buffer) == 1)
     {
       hb_buffer_clear (buffer);
       G_UNLOCK (cached_buffer);
     }
   else
-    hb_buffer_free (buffer);
+    hb_buffer_destroy (buffer);
 }
 
 /**
@@ -155,6 +155,8 @@ pango_ot_buffer_set_rtl (PangoOTBuffer *buffer,
 			 gboolean       rtl)
 {
   buffer->rtl = rtl != FALSE;
+  hb_buffer_set_direction (buffer->buffer,
+			   buffer->rtl ? HB_DIRECTION_RTL : HB_DIRECTION_LTR);
 }
 
 /**
@@ -195,10 +197,10 @@ pango_ot_buffer_get_glyphs (const PangoOTBuffer  *buffer,
 			    int                  *n_glyphs)
 {
   if (glyphs)
-    *glyphs = (PangoOTGlyph *)buffer->buffer->in_string;
+    *glyphs = (PangoOTGlyph *) hb_buffer_get_glyph_infos (buffer->buffer);
 
   if (n_glyphs)
-    *n_glyphs = buffer->buffer->in_length;
+    *n_glyphs = hb_buffer_get_len (buffer->buffer);
 }
 
 static void
@@ -222,9 +224,9 @@ swap_range (PangoGlyphString *glyphs, int start, int end)
 }
 
 static void
-apply_gpos_ltr (PangoGlyphString *glyphs,
-		HB_Position      positions,
-		gboolean         is_hinted)
+apply_gpos_ltr (PangoGlyphString    *glyphs,
+		hb_glyph_position_t *positions,
+		gboolean             is_hinted)
 {
   int i;
 
@@ -264,9 +266,9 @@ apply_gpos_ltr (PangoGlyphString *glyphs,
 }
 
 static void
-apply_gpos_rtl (PangoGlyphString *glyphs,
-		HB_Position      positions,
-		gboolean         is_hinted)
+apply_gpos_rtl (PangoGlyphString    *glyphs,
+		hb_glyph_position_t *positions,
+		gboolean             is_hinted)
 {
   int i;
 
@@ -325,25 +327,30 @@ pango_ot_buffer_output (const PangoOTBuffer *buffer,
 			PangoGlyphString    *glyphs)
 {
   FT_Face face;
-  PangoOTInfo *info;
-  HB_GDEF gdef = NULL;
+  hb_face_t *hb_face;
   unsigned int i;
   int last_cluster;
 
+  unsigned int len;
+  PangoOTGlyph *otglyphs;
+  hb_glyph_position_t *positions;
+
   face = pango_fc_font_lock_face (buffer->font);
   g_assert (face);
 
+  pango_ot_buffer_get_glyphs (buffer, &otglyphs, (int *) &len);
+
   /* Copy glyphs into output glyph string */
-  pango_glyph_string_set_size (glyphs, buffer->buffer->in_length);
+  pango_glyph_string_set_size (glyphs, len);
 
   last_cluster = -1;
-  for (i = 0; i < buffer->buffer->in_length; i++)
+  for (i = 0; i < len; i++)
     {
-      HB_GlyphItem item = &buffer->buffer->in_string[i];
+      PangoOTGlyph *otglyph = &otglyphs[i];
 
-      glyphs->glyphs[i].glyph = item->gindex;
+      glyphs->glyphs[i].glyph = otglyph->glyph;
 
-      glyphs->log_clusters[i] = item->cluster;
+      glyphs->log_clusters[i] = otglyph->cluster;
       if (glyphs->log_clusters[i] != last_cluster)
 	glyphs->glyphs[i].attr.is_cluster_start = 1;
       else
@@ -352,8 +359,7 @@ pango_ot_buffer_output (const PangoOTBuffer *buffer,
       last_cluster = glyphs->log_clusters[i];
     }
 
-  info = pango_ot_info_get (face);
-  gdef = pango_ot_info_get_gdef (info);
+  hb_face = _pango_ot_info_get_hb_face (pango_ot_info_get (face));
 
   /* Apply default positioning */
   for (i = 0; i < (unsigned int)glyphs->num_glyphs; i++)
@@ -362,12 +368,8 @@ pango_ot_buffer_output (const PangoOTBuffer *buffer,
 	{
 	  PangoRectangle logical_rect;
 
-	  HB_UShort property;
-
 	  if (buffer->zero_width_marks &&
-	      gdef &&
-	      HB_GDEF_Get_Glyph_Property (gdef, glyphs->glyphs[i].glyph, &property) == HB_Err_Ok &&
-	      (property == HB_GDEF_MARK || (property & HB_LOOKUP_FLAG_IGNORE_SPECIAL_MARKS) != 0))
+	      hb_ot_layout_get_glyph_class (hb_face, glyphs->glyphs[i].glyph) == HB_OT_LAYOUT_GLYPH_CLASS_MARK)
 	    {
 	      glyphs->glyphs[i].geometry.width = 0;
 	    }
@@ -390,12 +392,13 @@ pango_ot_buffer_output (const PangoOTBuffer *buffer,
       swap_range (glyphs, 0, glyphs->num_glyphs);
     }
 
+  positions = hb_buffer_get_glyph_positions (buffer->buffer);
   if (buffer->applied_gpos)
     {
       if (buffer->rtl)
-	apply_gpos_rtl (glyphs, buffer->buffer->positions, buffer->font->is_hinted);
+	apply_gpos_rtl (glyphs, positions, buffer->font->is_hinted);
       else
-	apply_gpos_ltr (glyphs, buffer->buffer->positions, buffer->font->is_hinted);
+	apply_gpos_ltr (glyphs, positions, buffer->font->is_hinted);
     }
   else
     {
